// ---------------------------------------------------------------------------------------------------
// Programa Base para el uso del modulo Sonoff Basic con Virtuino
// Gobierna el rele a traves del canal DV0. 
// El Led parpadea mientras intenta conectar a la WiFi y una vez conectado sigue al estado del rele.
// Tambien hace un pequeño flasheo ciclico cada 2s al estado contrario para saber que el modulo vive.
// A parte desde Virtuino tambien se puede gobernar manualmente con el pulsador y el estado se actualiza en el canal DV0 de Virtuino.
// ---------------------------------------------------------------------------------------------------
// Version 1.0 - Junio de 2017
// Diego Maroto. BilbaoMakers
// ---------------------------------------------------------------------------------------------------


// Las lineas con ** en el comentario son en las que tenemos que configurar cosas como la Wifi a conectar, la IP del cacharro y si queremos la contraseña y puerto del HTTP Server.


// LIBRERIAS
	#include <ESP8266WiFi.h>
	#include "Virtuino_ESP_WifiServer.h"
	
	
// SALIDAS
	#define LedVerde D7
	#define Rele D6
	#define AUX D5		// Esta salida no se usa para nada pero la tenemos disponible en el modulo Sonoff. Aqui de momento no esta implementada.

	
// ENTRADAS
	#define Pulsador D3
	
	
// CONSTANTES PARA LA CONFIGURACION WIFI
	const char* ssid = "SSID-MIWIFI";  			// ** SSID de la Wifi
	const char* password = "PASSWD-MIWIFI";		// ** Contraseña

	
// OBJETOS DEL SERVIDOR WIFI
	WiFiServer server(8000);					// ** Instanciar el servidor HTTP y puerto de escucha.
	Virtuino_ESP_WifiServer virtuino(&server);

	
// Variable interna para el estado del rele
	int storedValueRelay=0;						// Para almacenar el estado del rele para el tema del virtuino
	int EstadoRele = LOW;						// Para almacenar el estado del rele y hacer togle con el boton
	
// Variables para la lectura del pulsador
	int EstadoPulsador;          				// Para almacenar la ultima lectura 
	int UltimoEstadoPulsador = LOW;   			// Para almacenar el estado anterior
	
// Para la gestion de los reobotes
	unsigned long TiempoUltimoCambio = 0;  		// El tiempo del ultimo cambio en el estado del boton (lo llenaremos con la funcion millis).
	unsigned long DelayRebote = 50;   	 		// Tiempo Antirrebote (ms)
	
// Para gestion del led mientras conecta
	int EstadoConexion = HIGH;
	
// Para la gestion del flasheo del led para saber que el bicho esta vivo
	unsigned long TiempoFlash = 0;
	unsigned long TiempoUltimoFlash = 0;
	int EstadoLed = 0;

void setup() {
	
// Configuracion de Virtuino
  virtuino.DEBUG=true;                         	// Monitor de estado puerto serie.
  virtuino.password="1234";                    	// ** Password para el servidor HTTP. Solo numeros y caracteres standard.
                                               
  Serial.begin(9600);                          	// Arrancar el puerto serie para el Debug.
  delay(10);

// Configurar los pines del LED y el Rele
  pinMode(Rele,OUTPUT);            				// Pin de gobierno del Rele.
  pinMode(LedVerde,OUTPUT);             		// Pin de gobierno del LED.

// Configuracion y conexion a la WIFI
  
  Serial.println("Conectando a la wifi "+String(ssid));
  
  IPAddress ip(192, 168, 1, 200);      			// ** Direccion IP para el modulo Sonoff.
  IPAddress gateway(192, 168, 1, 1);   			// ** Puerta de enlace de nuestra red.
  IPAddress subnet(255, 255, 255, 0);        	// ** Mascara de red.
  WiFi.config(ip, gateway, subnet);          	// Configuracion. Para DHCP deshabilitar esta linea.
  
  WiFi.mode(WIFI_STA);                     		// Configurar como cliente wifi.
  WiFi.begin(ssid, password);
  Serial.print("Intentando conectar a la red ");// Escribir un mensajito por la consola
   while (WiFi.status() != WL_CONNECTED) {		// Esperar en un bucle a que la propiedad del objeto WiFi.status cambie a WL_CONNECTED
	 EstadoConexion=!EstadoConexion	;			// Parpadear el rele mientras conecta.
	 digitalWrite(LedVerde,EstadoConexion);
     delay(500);
     Serial.print("."); 						// Y mientras estamos ahi ir escribiendo puntitos por la consola
    }
   Serial.println("");							// Cuando el estado ya es WL_CONNECTED seguir ....
   Serial.println("WiFi conectada");			// Y escribir mensaje y la IP que tengo
   Serial.println(WiFi.localIP());				
	
   
// Arrancar el servidor HTTP que conecta con Virtuino
  server.begin();
  Serial.println("Servidor HTTP iniciado");

  
// Poner las salidas en un estado conocido (apagadas, que como van con MOSFET es a 1) 
  digitalWrite(LedVerde,1); 					
  digitalWrite(Rele,1); 						

// Igualar el estado del rele con el del Led
  EstadoLed = EstadoRele;
	
}


void loop() {
	
  
  // ALGORITMO PARA GESTIONAR EL VALOR QUE VIENE DE VIRTUINO
  
  virtuino.run();									  		// Esta es la funcion principal de la libreria de Arduino. Hace un ciclo de comunicaciones con el control.

  int v=virtuino.vDigitalMemoryRead(0);              		// Leer el VM0 de Virtuino
   
   if (v!=storedValueRelay) {                         		// Si el valor leido es contrario al estado actual del rele ....
    Serial.println("Rele cambiado al estado"+String(v));
    
      if (v==1) {									  		// Si es 1, cambiar la variable a LOW (el rele se enciende con el pin a LOW)
        
		EstadoRele = LOW;
      
	  }
    
      else {										  		// Y si no a HIGH (apagar el rele)
        
		EstadoRele = HIGH;
      
	  }
    
    storedValueRelay=v;										// Y almacenar el nuevo estado para comprobar si cambia en el proximo ciclo
   
   }

    
   // ALGORITMO PARA GESTION DEL PULSADOR
   
	int w=digitalRead(Pulsador);							// Leer el estado de la entrada del pulsador

	if (w != UltimoEstadoPulsador) {						// Si es diferente al anterior
    
		TiempoUltimoCambio = millis();						// Anotar el tiempo
	
	}
      
	if ((millis() - TiempoUltimoCambio) > DelayRebote) {	// Si ha pasado ya el tiempo estipulado antirebotes ...
       
		if (w != EstadoPulsador) {							// Comprobar si el boton cambia de estado de nuevo (
			
			EstadoPulsador = w;

			if (EstadoPulsador == HIGH) {					// Y si ese estado es HIGH (en el caso del boton es que lo he soltado, o sea que ya he hecho un ciclo de apretar + 50ms + soltar ....)
			
				EstadoRele = !EstadoRele;					// Cambiar la variable interna EstadoRele a lo contrario de lo que tenga
				Serial.println("Rele cambiado al estado"+String(EstadoRele));
			
			}
    
		}
  
	}
   
   	UltimoEstadoPulsador = w; // Actualizar la variable del ultimo estado leido del pulsador para volver a comprobar si lo apreto o suelto de nuevo en el proximo ciclo
	
			
	// Flasear un poquito el LED al estado contrario para saber que el modulo esta vivo.
   
	TiempoFlash = millis();
   
    if((TiempoFlash - TiempoUltimoFlash) > 2000 ) {
		
		EstadoLed = !EstadoRele;
		
		if ((TiempoFlash - TiempoUltimoFlash) > 2100) {
			
			EstadoLed = EstadoRele;
			TiempoUltimoFlash=millis();
			
		}
	
	
	}
	
	else {
		
		EstadoLed = EstadoRele;
		
	}
	
	
			
   // Escribir en las salidas rele, Led y en el canal digital de virtuino el estado de la variable (en virtuino le damos la vuelta porque aqui la variable va al reves por el MOSFET del rele).
   
   	digitalWrite(Rele,!EstadoRele);						
    digitalWrite(LedVerde,EstadoLed);
  	virtuino.vDigitalMemoryWrite(0, !EstadoRele);
  
}


